"""
股票市场风格因子离散度
"""
import os
import pandas as pd
import hbshare as hbs
from datetime import datetime, timedelta
from hbshare.quant.Kevin.rm_associated.config import style_names, industry_names
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB

path = r'D:\kevin\risk_model_jy\RiskModel\data'


class StyleFactorSpread:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_style_spread'
        self._load_calendar()

    def _load_calendar(self):
        pre_dt = datetime.strptime(self.start_date, '%Y-%m-%d') - timedelta(days=60)
        pre_date = pre_dt.strftime('%Y%m%d')
        end_date = self.end_date.replace('-', '')
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df

    @staticmethod
    def cal_spread(series):
        bottom = series[series <= series.quantile(0.2)].median()
        top = series[series >= series.quantile(0.8)].median()
        spread = top - bottom

        return spread

    def get_factor_spread(self):
        trading_day_list = self.calendar_df[self.calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()
        init_spread_list = []
        for date in trading_day_list:
            # 本地的风格因子数据
            data_path = os.path.join(path, r'zzqz_sw\style_factor')
            style_factor = pd.read_csv(
                os.path.join(data_path, '{0}.csv'.format(date)), dtype={"ticker": str}).set_index(
                'ticker')
            industry = pd.melt(style_factor[industry_names.values()].reset_index(),
                               id_vars=['ticker'], var_name='industry', value_name='sign')
            industry = industry[industry['sign'] == 1]
            merged_df = style_factor[style_names].merge(
                industry.set_index('ticker')['industry'], left_index=True, right_index=True).reset_index(drop=True)
            t_spread = []
            for name in style_names:
                t_spread.append(merged_df.groupby('industry')[name].apply(lambda df: self.cal_spread(df)).mean())
            spread = pd.Series(index=style_names, data=t_spread).to_frame(date)
            init_spread_list.append(spread)

        spread_df = pd.concat(init_spread_list, axis=1).T.sort_index()
        spread_df.index.name = 'trade_date'
        spread_df.reset_index(inplace=True)

        return spread_df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_factor_spread()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_stock_style_spread(
                id int auto_increment primary key,
                trade_date date not null unique,
                size decimal(4, 3),
                beta decimal(4, 3),
                momentum decimal(4, 3),
                earnyield decimal(4, 3),
                resvol decimal(4, 3),
                growth decimal(4, 3),
                btop decimal(4, 3),
                leverage decimal(4, 3),
                liquidity decimal(4, 3),
                sizenl decimal(4, 3))
            """
            create_table(self.table_name, sql_script)
            data = self.get_factor_spread()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StyleFactorSpread('2010-01-01', '2021-05-17', is_increment=0).get_construct_result()


"""
Zep风格因子模块
"""
import pandas as pd
from sqlalchemy import create_engine
from datetime import datetime, timedelta
import pyecharts.options as opts
from pyecharts.charts import Line, Bar
from pyecharts.globals import ThemeType
import hbshare as hbs
from hbshare.quant.Kevin.rm_associated.config import engine_params, style_names, industry_names


class RiskFactorReturn:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    @staticmethod
    def fetch_data_batch_hbs(user_name, sql_script):
        total_res = hbs.db_data_query(user_name, sql_script, is_pagination=False)
        n = total_res['pages']
        all_data = []
        for i in range(1, n + 1):
            res = hbs.db_data_query(
                user_name, sql_script, page_num=i, is_pagination=True, page_size=total_res['pageSize'])
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def _load_data(self):
        trade_dt = datetime.strptime(self.start_date, '%Y%m%d')
        pre_date = (trade_dt - timedelta(days=60)).strftime('%Y%m%d')
        sql_script = "SELECT * FROM factor_return where TRADE_DATE >= {} and TRADE_DATE <= {}".format(pre_date,
                                                                                                      self.end_date)
        engine = create_engine(engine_params)
        factor_return = pd.read_sql(sql_script, engine)
        factor_return['trade_date'] = factor_return['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        factor_return = pd.pivot_table(
            factor_return, index='trade_date', columns='factor_name', values='factor_ret').sort_index()
        factor_return = factor_return[style_names + list(industry_names.values()) + ['country']]

        self.factor_return = factor_return[factor_return.index >= self.start_date]
        self.factor_return_pre = factor_return

    def generate_trendency_line(self, title):
        factor_return = self.factor_return.copy()
        factor_return = factor_return.cumsum()[style_names] * 100
        cum_line = Line(
            init_opts=opts.InitOpts(
                page_title=title,
                width='1200px',
                height='600px',
                theme=ThemeType.WALDEN
            )
        ).set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            legend_opts=opts.LegendOpts(legend_icon='roundRect', pos_top='5%'),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axistick_opts=opts.AxisTickOpts(is_show=True, is_align_with_label=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_interval=0.1,
                name='Cumulative Performance(%)',
                name_location='middle',
                name_gap=45,
                name_textstyle_opts=opts.TextStyleOpts(font_size=16),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
        ).add_xaxis(
            xaxis_data=factor_return.index.tolist()
        )

        for style_factor in style_names:
            cum_line.add_yaxis(
                series_name=style_factor,
                is_smooth=True,
                y_axis=factor_return[style_factor].round(4).tolist(),
                label_opts=opts.LabelOpts(is_show=False)
            )

        html_content = cum_line.render_embed()
        print("%html {}".format(html_content))

    def generate_factor_volatility(self, title, window=20):
        factor_return = self.factor_return_pre.copy()
        factor_return = factor_return[style_names] * 100
        factor_vol = factor_return.rolling(window).std().dropna()
        factor_vol = factor_vol[factor_vol.index >= self.start_date]

        line = Line(
            init_opts=opts.InitOpts(
                page_title=title,
                width='1200px',
                height='600px',
                theme=ThemeType.WALDEN
            )
        ).set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            legend_opts=opts.LegendOpts(legend_icon='roundRect', pos_top='5%'),
            tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axistick_opts=opts.AxisTickOpts(is_show=True, is_align_with_label=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_interval=0.1,
                name='Factor Volatility(%)',
                name_location='middle',
                name_gap=45,
                name_textstyle_opts=opts.TextStyleOpts(font_size=16),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
        ).add_xaxis(
            xaxis_data=factor_vol.index.tolist()
        )

        for style_factor in style_names:
            line.add_yaxis(
                series_name=style_factor,
                is_smooth=True,
                y_axis=factor_vol[style_factor].round(4).tolist(),
                label_opts=opts.LabelOpts(is_show=False)
            )

        html_content = line.render_embed()
        print("%html {}".format(html_content))

    def generate_period_cumret_bar(self, title):
        factor_return = self.factor_return.copy()
        factor_return = (factor_return.sum()[style_names] * 100).round(2)

        bar = Bar(
            init_opts=opts.InitOpts(
                page_title=title,
                width='1200px',
                height='600px',
                theme=ThemeType.LIGHT
            )
        ).set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            legend_opts=opts.LegendOpts(legend_icon='roundRect', pos_top='5%'),
            tooltip_opts=opts.TooltipOpts(
                is_show=True, trigger="axis", axis_pointer_type="cross"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axislabel_opts={"interval": "0"},
                axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_interval=0.1,
                name='Cumulative Performance(%)',
                name_location='middle',
                name_gap=45,
                name_textstyle_opts=opts.TextStyleOpts(font_size=16),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
        ).add_xaxis(
            xaxis_data=["市值", "Beta", "动量", "盈利", "波动率", "成长", "估值", "杠杆", "流动性", "非线性市值"]
        ).add_yaxis(
            series_name='因子累计收益',
            y_axis=factor_return.values.tolist(),
            bar_width="60%"
        )

        html_content = bar.render_embed()
        print("%html {}".format(html_content))

    def generate_period_industry_bar(self, title):
        factor_return = self.factor_return.copy()
        industry = dict([val, key] for key, val in industry_names.items())
        df = (factor_return.sum().loc[list(industry.keys())] * 100).round(2)
        df.index = [industry[x] for x in df.index]
        df.sort_values(ascending=False, inplace=True)

        bar = Bar(
            init_opts=opts.InitOpts(
                page_title=title,
                width='1200px',
                height='600px',
                theme=ThemeType.LIGHT
            )
        ).set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            legend_opts=opts.LegendOpts(legend_icon='roundRect', pos_top='5%'),
            tooltip_opts=opts.TooltipOpts(
                is_show=True, trigger="axis", axis_pointer_type="cross"),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axislabel_opts={"interval": "0", "rotate": "90"},
                axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_interval=0.1,
                name='Cumulative Performance(%)',
                name_location='middle',
                name_gap=45,
                name_textstyle_opts=opts.TextStyleOpts(font_size=16),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
        ).add_xaxis(
            xaxis_data=df.index.tolist()
        ).add_yaxis(
            series_name='行业因子累计收益',
            y_axis=df.values.tolist(),
            bar_width="60%"
        )

        html_content = bar.render_embed()
        print("%html {}".format(html_content))


if __name__ == '__main__':
    RiskFactorReturn(start_date='20220101', end_date='20220222').generate_factor_volatility(title="Factor Volatility")


if __name__ == '__main__':

    from hbshare.quant.CChen.stk import index_win, load_index, load_stk_local, ch_stk_quote_to_local
    import pandas as pd
    from db_cons import sql_write_path_work, sql_user_work
    from datetime import datetime, timedelta

    index_list = [
        '000300',
        '000905',
        '000852'
    ]

    freq_list = [
        'D',
        'W',
        'M',
    ]

    end_date = datetime.now().date()
    # end_date = datetime(2019, 1, 1).date()

    table_index_win = 'index_win'

    ch_stk_quote_to_local(db_path=sql_write_path_work['daily'], sql_info=sql_user_work)

    for i in freq_list:
        last_date = pd.read_sql_query(
            'select TDATE from ' + table_index_win
            + ' where FREQ=\'' + i + '\' order by TDATE desc limit 1',
            sql_write_path_work['daily']
        )

        if len(last_date) == 0:
            last_date = datetime(2010, 1, 1).date()
        else:
            last_date = last_date['TDATE'][0] + timedelta(days=1)

        index_data = load_index(index_list=index_list, start_date=last_date, end_date=end_date)
        stk_data = load_stk_local(
            start_date=last_date, end_date=end_date, sql_path=sql_write_path_work['daily']
        ).rename(columns={'SYMBOL': 'CODE'})

        index_win_data = index_win(
            index_list=index_list,
            freq=i,
            index_data=index_data,
            stk_data=stk_data
        )
        if index_win_data is not None:
            if len(index_win_data) > 0:
                index_win_data['FREQ'] = i
                index_win_data.to_sql(table_index_win, sql_write_path_work['daily'], if_exists='append', index=False)
                print(i + ' ' + end_date.strftime('%Y/%m/%d') + ' done, ' + str(len(index_win_data)))




"""
股票指数时序波动率指标
"""
import pandas as pd
from datetime import datetime, timedelta
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class StockTimeSeriesVol:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_time_series_vol'

    def get_time_series_vol(self):
        """
        股票市场指数行情数据：包含沪深300、中证500、中证1000、万得全A
        """
        index_list = ['000300.SH', '000905.SH', '000852.SH', '881001.WI']
        pre_dt = datetime.strptime(self.start_date, '%Y-%m-%d') - timedelta(days=50)
        pre_date = pre_dt.strftime('%Y-%m-%d')
        res = w.wsd(','.join(index_list), 'close', pre_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch pe_ttm data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        index_return = data.set_index('trade_date').pct_change().dropna()
        # 5日
        tmp = index_return.rolling(5).std()
        vol_5 = tmp[tmp.index >= self.start_date.replace('-', '')]
        vol_5.rename(columns={"000300.SH": "HS300_5d", "000905.SH": "ZZ500_5d",
                              "000852.SH": "ZZ1000_5d", "881001.WI": "WindA_5d"}, inplace=True)
        # 20日
        tmp = index_return.rolling(20).std()
        vol_20 = tmp[tmp.index >= self.start_date.replace('-', '')]
        vol_20.rename(columns={"000300.SH": "HS300_20d", "000905.SH": "ZZ500_20d",
                               "000852.SH": "ZZ1000_20d", "881001.WI": "WindA_20d"}, inplace=True)
        df = pd.merge(vol_5, vol_20, left_index=True, right_index=True).reset_index()

        return df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_time_series_vol()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_stock_time_series_vol(
                id int auto_increment primary key,
                trade_date date not null unique,
                HS300_5d decimal(5, 4),
                ZZ500_5d decimal(5, 4),
                ZZ1000_5d decimal(5, 4),
                WindA_5d decimal(5, 4),
                HS300_20d decimal(5, 4),
                ZZ500_20d decimal(5, 4),
                ZZ1000_20d decimal(5, 4),
                WindA_20d decimal(5, 4)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_time_series_vol()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockTimeSeriesVol('2021-04-23', '2021-04-23', is_increment=1).get_construct_result()


import os
import pandas as pd
import datetime
from sqlalchemy import create_engine
import hbshare as hbs
from ..XZ.config import config_pfa

configuration=config_pfa.Config()

class PrvFunDB:

    def __init__(self):

        self.sql_info= configuration.DBconfig
        self.columns_name_trans = configuration.Columns_name_trans
        self.mkr_code_dic = configuration.Mkr_code

        #set up the engine
        self.engine= create_engine("mysql+pymysql://{}:{}@{}:{}/{}".format(
            self.sql_info['Sql_user'], self.sql_info['Sql_pass'],
            self.sql_info['Sql_ip'], self.sql_info['port'], self.sql_info['Database']))

    def read_files_2_df(self,root_dir,dirs):

        # initial the output table
        value_table = pd.DataFrame()
        fold_dir = root_dir + "\\" + dirs
        for root, dir, files in os.walk(fold_dir, topdown=False):
            file_list = files

        # go through all the fiels in the sub folder
        for files in file_list:

            # read the file and rename the columns
            temp_excel = pd.read_excel(fold_dir + "\\" + files, header=3)

            # check if there is any regular columns missed if yes add those columns
            if (len(list(set(self.columns_name_trans.keys()).difference(set(temp_excel.columns)))) > 0):
                for col in list(set(self.columns_name_trans.keys()).difference(set(temp_excel.columns))):
                    temp_excel[col] = None

            # rename all the chinese column names
            temp_excel.rename(columns=self.columns_name_trans, inplace=True)

            # add the column  stamp date and product name based on the file names,the frequence based on data
            Stamp_date = ''.join(filter(lambda x: x.isdigit(), files.split('_')[2].split('.')[0]))

            temp_excel = temp_excel[~(temp_excel['Code'].isnull())]

            temp_excel['Stamp_date'] = datetime.datetime.strptime(Stamp_date, "%Y%m%d").date()
            temp_excel['Prd_name'] = dirs
            temp_excel['Fre'] = 'Monthly'

            # combine all the tables into one output table
            value_table = pd.concat([value_table, temp_excel], axis=0)

        return value_table

    def create_new_table(engine):

        # generate the sql for creating the table
        sql = """
        CREATE TABLE PrvFund (
                 Code  CHAR(100),
                 Name  CHAR(100),
                 Quant FLOAT,  
                 Unit_cost FLOAT,
                 Cost FLOAT,
                 Cost_weight FLOAT,
                 Price FLOAT,
                 Mkt_value FLOAT,
                 Weight FLOAT,
                 Increased_val FLOAT,
                 Trading_flg CHAR(20),
                 Currency  CHAR(20),
                 Hd_info CHAR(100),
                 Ex_rate FLOAT,
                 Stamp_date timestamp ,
                 Prd_name CHAR(20),
                 Fre CHAR(20)
                 )
                 """

        # delect the will-be-creted table if exist already
        with engine.connect() as con:
            con.execute("DROP TABLE IF EXISTS PrvFund")
            con.execute(sql)

    def delete_table(self):

        # delect the will-be-creted table if exist already
        with self.engine.connect() as con:
            con.execute("DROP TABLE IF EXISTS {}".format(self.table_name))

    def write2DB(self,table_name,fold_dir,increment):

        # self.delete_table()

        # get all the sub fold names in the root folder
        for root, dir, files in os.walk(fold_dir,topdown=False):
            dir_list = dir

        # for each sub folders  go through all the fiels in the it
        for dirs in dir_list:

            # save file data into dataframe
            Input_table = self.read_files_2_df(fold_dir,dirs)

            # if not increment out-dated data need to be deleted first
            if (increment == 0):

                # collect all the stamp_date that need to be deleted
                Date_list = Input_table['Stamp_date'].unique().astype('str')

                # set up the sql to count the number of data for the target stamp date
                count_sql = """ 
                    select count(*) as count from {0} where Prd_name= '{1}' and Stamp_date in ({2}) 
                """.format(table_name, dirs, ','.join(["'%s'" % item for item in Date_list]))
                count = pd.read_sql(sql=count_sql, con=self.engine)['count'][0]

                # if there is out-dated data
                if (count > 0):
                    delete_sql = """ 
                        delete from {0} where Prd_name= '{1}' and Stamp_date in ({2}) 
                    """.format(table_name, dirs, ','.join(["'%s'" % item for item in Date_list]))
                    with self.engine.connect() as con:
                        con.execute(delete_sql)

            # write dataframe into database
            Input_table.to_sql(table_name, self.engine, index=False, if_exists='append')

    def get_trading_date(self,date_list):

        last_date = date_list[-1].strftime('%Y%m%d')
        pre_date = (date_list[-0] - datetime.timedelta(days=30)).strftime('%Y%m%d')

        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, last_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        calander = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        calander['isOpen'] = calander['isOpen'].astype(int).replace({0: 1, 1: 0})
        calander['isWeekEnd'] = calander['isWeekEnd'].fillna(0).astype(int)
        calander['isMonthEnd'] = calander['isMonthEnd'].fillna(0).astype(int)
        #
        trading_day_list = calander[calander['isOpen'] == 1]['calendarDate'].tolist()
        trading_day_list=[ datetime.datetime.strptime(x, "%Y%m%d").date() for x in  trading_day_list]

        return  trading_day_list

    def _shift_df_date(self,df):

        date_list = df['Stamp_date'].dropna().unique()
        trading_day_list=self.get_trading_date(date_list)

        missing_date=set(date_list).difference(set( trading_day_list))
        if(len(missing_date)>0):
            for missed_date in missing_date:
                delta_date=[x-missed_date for x in trading_day_list]
                df.loc[df['Stamp_date'] == missed_date, 'Stamp_date'] = trading_day_list[
                    [abs(x) for x in delta_date] == min([abs(x) for x in delta_date])]

        return df

    def extract_from_db(self,prd_name,columns,tablename):

        #read the table from the temp database
        sql="""
            select {0} from {1} where Prd_name= '{2}'
        """.format(columns,tablename,prd_name)
        df=pd.read_sql(sql,con=self.engine)

        #add a new columd stock_code by translating code
        df['Stock_code'] = None
        for mkr_code in self.mkr_code_dic.values():
            df=self.generate_stockcode_from_code(df,mkr_code)

        #transfor datetime format to string format
        # df['Stamp_date']=[x.strftime("%Y-%m-%d") for x in df['Stamp_date']]

        return self._shift_df_date(df)

    def read_from_db(self,sql):
        df = pd.read_sql(sql, con=self.engine)
        return  df

    def generate_stockcode_from_code(self,df,code):

        df.loc[df['Code'].str.startswith(code), 'Stock_code'] = \
        df[(df['Code'].str.startswith(code)) & (df['Code'] !=code)]['Code'] \
            .apply(lambda x: x.split(code)[1])
        return df

class HBDB:

    def __init__(self):

        self.sql_info = configuration.DBconfig
        self.columns_name_trans = configuration.Columns_name_trans
        self.jy_sql = configuration.Jy_sql

        hbs.set_token("")

    def db2df(self,sql,db='readonly'):

        data = hbs.db_data_query(db, sql, page_size=2000,timeout=120)
        pages=data['pages']
        data = pd.DataFrame(data['data'])
        if(pages>1):
            for page in range(2,pages+1):
                temp_data= hbs.db_data_query(db, sql, page_size=2000,page_num=page,timeout=120)
                data=pd.concat([data,pd.DataFrame(temp_data['data'])],axis=0)
        return data

    def extract_industry(self,sec_code=None):

        if(sec_code is None ):
            #join the table with company stock code and company industry
            #sql1 for normal stock sql2 for kechuang ban stock
            sql1= self.jy_sql['sql_indus_a']['sql'].replace('@@','')
            sql2 = self.jy_sql['sql_indus_b']['sql'].replace('@@','')
        else:
            sql1 = self.jy_sql['sql_indus_a']['sql']\
                .replace('@@', "and A.SecuCode in ("+','.join(["'%s'" % item for item in sec_code])+")")
            sql2 = self.jy_sql['sql_indus_b']['sql']\
                .replace('@@', "and A.SecuCode in (" + ','.join(["'%s'" % item for item in sec_code]) + ")")

        df1 = self.db2df(sql1)
        df2 = self.db2df(sql2)
        data=pd.concat([df1,df2],axis=0)

        #some of the stocks has multiple industries,keep the latest one row
        data = data.sort_values(by='XGRQ', ascending=False)
        data.drop_duplicates(subset=['SECUCODE', 'SECUABBR', 'COMPANYCODE'],inplace=True,keep='first')
        return data[['SECUCODE', 'SECUABBR', 'COMPANYCODE','FIRSTINDUSTRYNAME']]

    def extract_fin_info(self,sec_code=None,date_list=None):

        input_date_list=date_list.copy()
        if(input_date_list is None):
            datalistcon=''
        else:
            # date_list_str = [''.join(str(x).split('-')) for x in input_date_list]
            # sql=self.jy_sql['sql_fin_date']['sql'].format(date_list_str[0],date_list_str[-1])
            #
            # date_df=self.db2df(sql=sql,db='readonly')
            # date_df['TRADINGDAY'] = [datetime.datetime.strptime(x, "%Y-%m-%d").date() for x in
            #                       (date_df['TRADINGDAY'].apply(lambda x: x[0:10]))]
            # missing_date=list( set(input_date_list).difference(set(date_df['TRADINGDAY'])) )
            #
            # for date in missing_date:
            #     date_delta =date_df['TRADINGDAY'] - date
            #     input_date_list[input_date_list==date]=date_df['TRADINGDAY'][date_delta.abs()==date_delta.abs().min()]
            #
            # input_date_list=list(set(input_date_list))
            date_list_str=[''.join(str(x)) for x in input_date_list]
            datalistcon="and a.TradingDay in ({})".format(','.join([ "to_date('"+x+"', 'yyyy-mm-dd')" for x in date_list_str]) )

        if(sec_code is None ):
            #join the table with company stock code and company industry
            sql = self.jy_sql['sql_fin']['sql'].replace('@@','')+datalistcon
        else:
            #filter the sql by security code range
            sql = self.jy_sql['sql_fin']['sql']\
                .replace('@@', "where b.SecuCode in (" + ','.join(["'%s'" % item for item in sec_code])+")")\
                  +datalistcon

        data=self.db2df(sql)

        # transfor tradingday format from yyyy-mm-dd hh-mm-ss to yyyy-mm-dd
        data['TRADINGDAY'] = [datetime.datetime.strptime(x, "%Y-%m-%d").date() for x in
                             (data['TRADINGDAY'].apply(lambda x: x[0:10]))]

        return data

    def extract_benchmark(self,benchmark_dict,sec_code=None,date_list=None):

        input_date_list=date_list.copy()
        if(input_date_list is None):
            datacond=''
        else:
            date_list_str = [''.join(str(x).split('-')) for x in input_date_list]
            datacond=" and ENDDATE>= to_date('{0}','yyyymmdd') and ENDDATE <=to_date('{1}','yyyymmdd') " .format(date_list_str[0], date_list_str[-1])


        if(sec_code is None ):
            sql = self.jy_sql['sql_benchmark']['sql'].replace('@@','')
        else:
            #filter the sql by security code range
            sql =self.jy_sql['sql_benchmark']['sql']\
                .replace('@@', "and b.SecuCode in (" + ','.join(["'%s'" % item for item in sec_code])+")")

        data=pd.DataFrame()
        for benchmarks in benchmark_dict.keys():
            sql2 = "SELECT INNERCODE FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(benchmark_dict[benchmarks])
            innercode = self.db2df(sql2)['INNERCODE'][0]
            tempdata=self.db2df(sql.replace('@IndexCode',str(innercode))+datacond)
            tempdata['Index_type']=benchmarks
            data=pd.concat([data,tempdata],axis=0)

        # transform tradingday format from yyyy-mm-dd hh-mm-ss to yyyy-mm-dd
        data['ENDDATE'] = [datetime.datetime.strptime(x, "%Y-%m-%d").date() for x in
                             (data['ENDDATE'].apply(lambda x: x[0:10]))]

        return data

    def extract_factors(self,sec_code,date_list,industry=False):

        # sql = """
        # select distinct( trade_date ) from st_ashare.r_st_barra_style_factor
        # where trade_date>= '{0}' and trade_date<= '{1}'
        # """.format(date_list[0],date_list[-1])
        #
        # available_date=self.db2df(sql, db='alluser')

        date_list=[''.join(x.split('-')) for x in date_list.astype(str)]
        # miss_date=set( date_list ).difference(set(available_date['trade_date']))
        #
        # for date in miss_date:
        #     date_delta = datetime.date(date_list) - datetime.date(date)
        #     date_list[date_list == date] = date_list[date_delta.abs() == date_delta.abs().min()]

        industry_col=['aerodef','agriforest','auto','bank','builddeco','chem','conmat','commetrade','computer','conglomerates',
                      'eleceqp','electronics','foodbever','health','houseapp','ironsteel','leiservice','lightindus',
                      'machiequip','media','mining','nonbankfinan','nonfermetal',
                    'realestate','telecom','textile','transportation','utilities']

        if(industry==True):
            industry_con=','+','.join(industry_col)
        else:
            industry_con=''

        sql="""
        select ticker,trade_date,size,beta,momentum,resvol,btop,sizenl,liquidity,earnyield,growth,leverage{2}
        from st_ashare.r_st_barra_style_factor where ticker in ({0}) and trade_date in ({1})
        """.format(','.join(sec_code),"'" + "','".join(date_list) + "'",industry_con)
        raw_factors = self.db2df(sql, db='alluser')

        if(industry==True):
            raw_factors[industry_col]=raw_factors[industry_col].astype(float)
            for col in industry_col:
                if(raw_factors[col].sum()==0):
                    raw_factors.drop(col,axis=1,inplace=True)

        return raw_factors

    def extract_benchmark_return(self,date_list,index_list):

        index_con="'"+"','".join(index_list)+"'"

        sql="""
        select ZQDM,TJYF,HB from funddb.T_FU_GG_ZSHB_M   
        where ZQDM in ({1}) and TJYF in ({0})
        """.format(','.join([ x[0:6] for x in [''.join(x.split('-')) for x in date_list.astype(str)]]),index_con)
        factors_benchmark=self.db2df(sql,'readonly')

        return  factors_benchmark


"""
基金风格箱的校验程序
"""
import pandas as pd
import hbshare as hbs


class MorningStarCheck:
    def __init__(self, trade_date, mode):
        self.trade_date = trade_date
        self.mode = mode
        self._get_report_date()

    def _get_report_date(self):
        pre_date = str(int(self.trade_date[:4]) - 2) + '0101'
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
           pre_date, self.trade_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
        df = df[df['isMonthEnd'] == 1]

        if self.mode == "main":  # quarter
            df['pre_end'] = df['calendarDate'].shift(1)
            quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'pre_end']
        else:
            df['shift_2'] = df['calendarDate'].shift(2)
            df['shift_4'] = df['calendarDate'].shift(4)
            if self.trade_date[4:6] == "08":
                quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'shift_2']
            else:
                quotation_date = df.set_index('calendarDate').loc[self.trade_date, 'shift_4']

        self.quotation_date = quotation_date

    def get_online_res_from_mysql(self):
        db_name = 'st_fund'
        user_name = 'funduser'

        # equity_score
        sql_script = "SELECT trade_date, ticker, type, cap_score, vcg_score, category FROM " \
                     "{}.r_st_gp_equity_score where trade_date = {} and " \
                     "type = '{}'".format(db_name, self.quotation_date, self.mode)
        res = hbs.db_data_query(user_name, sql_script, page_size=6000)
        equity_score = pd.DataFrame(res['data'])
        # equity_score = equity_score.drop_duplicates(subset=['ticker'])
        # fund_score
        sql_script = "SELECT trade_date, fund_id, type, cap_score, vcg_score, category FROM " \
                     "{}.r_st_gm_equity_score where trade_date = {} and " \
                     "type = '{}'".format(db_name, self.trade_date, self.mode)
        res = hbs.db_data_query(user_name, sql_script, page_size=6000)
        fund_score = pd.DataFrame(res['data'])
        # fund_score = fund_score.drop_duplicates(subset=['fund_id'])

        return equity_score, fund_score


if __name__ == '__main__':
    equity_score_online, fund_score_online = MorningStarCheck('20100430', 'all').get_online_res_from_mysql()
    print(equity_score_online)
    print(fund_score_online)


import  numpy as np
import pandas as pd
from hbshare.fe.XZ import  functionality
from hbshare.fe.XZ import db_engine as dbeng

util=functionality.Untils()
hbdb=dbeng.HBDB()
localdb=dbeng.PrvFunDB().engine

def create_stock_shift_ratio():

    hsl=pd.read_excel(r"E:\GitFolder\hbshare\fe\factor_analysis\换手率.xlsx")
    date_col=hsl.columns.tolist()
    date_col.remove('证券简称')
    date_col.remove('证券代码')
    new_date_col=[]
    date_map=dict()
    hsl['jjdm'] = hsl['证券代码'].str[0:6]
    for i in range(len(date_col)):
        new_date_col.append(date_col[i].replace('H1','0630').replace(' ','').replace('H2', '1231'))

    date_map = dict(zip(date_col,new_date_col))

    outputdf=pd.DataFrame()
    tempdf=pd.DataFrame()
    tempdf['jjdm']=util.get_mutual_stock_funds('20211231')
    tempdf=pd.merge(tempdf,hsl,how='left',on='jjdm')

    for date in date_col:
        tempdf2=tempdf[['jjdm',date]]
        tempdf2['date']=date_map[date]
        tempdf2.rename(columns={date:'hsl'},inplace=True)
        outputdf=pd.concat([outputdf,tempdf2],axis=0)

    outputdf=outputdf[outputdf['hsl'].notnull()]

    outputdf.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')

def read_factors(table_name):

    sql="select * from {}".format(table_name)
    df=pd.read_sql(sql,con=localdb)

    return  df

def bhar(arr):
    return 100 * (np.power(np.cumprod((arr + 100) / 100).tolist()[-1], 1 / len(arr)) - 1)


def hsl_rank2db(dir):

    #get the hsl raw data first
    # sql="select * from factor_hsl  "
    # df=pd.read_sql(sql,con=localdb)
    df=pd.read_csv(dir)
    df['hsl_rank']=df.groupby('date').rank(method='min')
    count=df.groupby('date', as_index=False).count()[['date', 'jjdm']]
    count.rename(columns={'jjdm':'count'},inplace=True)
    df=pd.merge(df,count,how='left',on='date')
    df['hsl_rank']=df['hsl_rank']/df['count']
    df.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')


if __name__ == '__main__':

    df=read_factors('factor_hsl')
    df=df.groupby('jjdm').min()
    df=df[df['hsl'] >= 350]

    jjdm_list=df.index.tolist()
    jjdm_con=util.list_sql_condition(jjdm_list)

    sql="""select jjdm,hb1n,rqzh from st_fund.t_st_gm_nhb 
    where hb1n!=99999 and jjdm in ({0}) and tjnf in ('2015','2016','2017','2018','2019','2020','2021')"""\
        .format(jjdm_con)
    ret=hbdb.db2df(sql,db='funduser')
    ret=ret.groupby('jjdm')['hb1n'].apply(bhar)
    ret.to_csv('hlsret.csv',encoding='gbk')
    print('')


"""
处理过的市场指数数据，用于添加在月度指标右轴
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class MarketIndex:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_market_index'

    def get_market_index_data(self):
        """
        沪深300、中证500、中债财富总指数、南华商品指数
        """
        index_list = ["000300.SH", "000905.SH", "H11001.CSI", "NH0100.NHF"]
        res = w.wsd(','.join(index_list), "close", self.start_date, self.end_date)

        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch market index data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            # 对齐到自然月末
            start_date, end_date = data.index[0], data.index[-1]
            data = data.reindex(pd.date_range(start_date, end_date)).fillna(method='ffill').reindex(
                pd.date_range(start_date, end_date, freq='M')).round(2)
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns={"000300.SH": "HS300", "000905.SH": "ZZ500",
                                 "H11001.CSI": "bond_index", "NH0100.NHF": "future_index"}, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_market_index_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                    create table mac_market_index(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    HS300 decimal(8, 2),
                    ZZ500 decimal(8, 2),
                    bond_index decimal(8, 2),
                    future_index decimal(8, 2)) 
                """
            create_table(self.table_name, sql_script)
            data = self.get_market_index_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    MarketIndex('2021-01-04', '2021-06-25', is_increment=1).get_construct_result()


"""
板块指数更新程序
"""
import pandas as pd
import numpy as np
from hbshare.fe.common.util.data_loader import SectorIndexCalculatorLoader
from hbshare.fe.common.util.config import industry_cluster_dict_new
from sqlalchemy import create_engine


class SectorIndexCalculator:
    def __init__(self, start_date, end_date, write2db=False):
        self.start_date = start_date
        self.end_date = end_date
        self.write2db = write2db
        self._load_data()

    def _load_data(self):
        self.industry_data = SectorIndexCalculatorLoader(self.start_date, self.end_date).load()

    def get_construct_result(self):
        industry_data = self.industry_data.copy()

        tmp = pd.pivot_table(
            industry_data, index='TRADEDATE', columns='INDEXNAME', values='TCLOSE').sort_index()
        industry_return = tmp.pct_change().dropna()
        industry_return = industry_return[industry_return.index >= self.start_date]

        tmp = pd.pivot_table(
            industry_data, index='TRADEDATE', columns='INDEXNAME', values='NEG_MKV').sort_index().fillna(method='ffill')
        industry_mkt = tmp.shift(1).dropna()
        industry_mkt = industry_mkt[industry_mkt.index >= self.start_date]

        assert(industry_return.shape == industry_mkt.shape)

        sector_return = []
        for key, value in industry_cluster_dict_new.items():
            weight = industry_mkt[value].divide(industry_mkt[value].sum(axis=1), axis='rows')
            ret_array = np.diag(weight.dot(industry_return[value].T))
            sector_series = pd.Series(index=industry_return.index, data=ret_array)
            sector_return.append(sector_series.to_frame(name=key))

        sector_return = pd.concat(sector_return, axis=1)

        sector_return = sector_return.reset_index().rename(
            columns={"大金融": "BIGFINANCE", "消费": "CONSUMING", "周期": "CYCLE", "制造": "MANUFACTURE"})

        if self.write2db:
            engine = create_engine("mysql+pymysql://{}:{}@{}:{}/{}".format(
                'admin', 'mysql', '192.168.223.152', '3306', 'riskmodel'))

            sector_return.to_sql('sector_return', engine, index=False, if_exists='append')

        return sector_return


if __name__ == '__main__':
    # res = SectorIndexCalculator('20210705', '20210820', write2db=True).get_construct_result()
    res = SectorIndexCalculator('20211201', '20211222').get_construct_result()
    print(res)


# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
from hbshare.fe.common.util.exception import PreprocessError
from hbshare.fe.common.util.logger import logger
from scipy.optimize import minimize


def linear_regression(n, x, y):
    A = np.array([x, np.ones(n)])
    w = np.linalg.lstsq(A.T, y)
    a = w[0][0]
    residual = y - x * a
    return w[0], residual


def delete_redundancy(x_data_frame, corr_threshold=0.7):
    corr_df = x_data_frame.corr()
    corr_len = len(corr_df)
    x_list = x_data_frame.columns.tolist()
    remove_list = []
    for i in range(corr_len):
        for j in range(i + 1, corr_len):
            if corr_df.iloc[i, j] > corr_threshold:
                remove_list.append(x_list[j])
                continue
    left_list = list(set(x_list) - set(remove_list))
    return x_data_frame[left_list]


class Regression:
    def __init__(self, x_data, y_data, upper, lower, ridge_lambda=0.001, method='ridge', effort='weak',
                 pca_vec_mat=None, use_pca=False):
        if method == "reduce":
            x_data = delete_redundancy(x_data)

        if 'alpha' not in x_data.columns:
            sort_list = ['alpha'] + x_data.columns.tolist()
            x_data.loc[:, 'alpha'] = 1.0
            x_data = x_data[sort_list]

        self.x_data = x_data
        self.y_data = y_data
        self.upper = upper
        self.lower = lower
        self.method = method
        self.effort = effort
        self.pca_vec_mat = pca_vec_mat
        self.use_pca = use_pca
        self.ridge_lambda = ridge_lambda

    def __ols_func(self, x):
        x_beta = (np.matrix(self.x_data) * np.matrix(x).T).A1
        eps = self.y_data - x_beta
        delta = sum(eps ** 2)
        return delta

    def __ridge_func(self, x):
        x_beta = (np.matrix(self.x_data) * np.matrix(x).T).A1
        eps = self.y_data - x_beta
        delta = sum(eps ** 2) + sum(x * x) * self.ridge_lambda
        return delta

    def __get_bnds(self):
        n_param = self.x_data.shape[1]
        x0 = np.ones(n_param) / float(n_param)

        if self.effort == 'hard':
            bnds = [(None, None)] + [(self.lower, self.upper) for _ in range(len(x0) - 1)]
            bnds = tuple(bnds)
        else:
            bnds = tuple((None, None) for _ in x0)
        return bnds, x0

    def __get_cons(self):
        if self.effort == 'hard':
            cons_list_dict = [{'type': 'eq', 'fun': lambda x: sum(x[1:]) - 1}]
        else:
            cons_list_dict = []

        if self.use_pca and self.pca_vec_mat is not None:
            for i in range(len(self.pca_vec_mat)):
                array_list = self.pca_vec_mat[i, :].A1
                cons_list_dict.append({'type': 'ineq', 'fun': lambda x: sum(x[1:] * array_list)})
        cons = tuple(cons_list_dict)
        return cons

    def solve(self):
        if not isinstance(self.x_data, (type(pd.DataFrame()), type(pd.Series()))):
            msg = "NavRegression x_data Type is not pandas.DataFrame and pandas.Series."
            logger.exception(msg)
            raise PreprocessError(message=msg)

        if not isinstance(self.y_data, type(pd.Series())):
            msg = "NavRegression y_data Type is note pandas.Series."
            logger.exception(msg)
            raise PreprocessError(message=msg)

        if len(self.x_data) != len(self.y_data):
            msg = "NavRegression x_data length not equals to y_data length."
            logger.exception(msg)
            raise PreprocessError(message=msg)

        if self.method == 'ridge':
            target_func = self.__ridge_func
        else:
            target_func = self.__ols_func

        bnds, x0 = self.__get_bnds()
        cons = self.__get_cons()
        options = {'disp': False, 'maxiter': 500, 'ftol': 1e-20}

        res = minimize(target_func, x0, bounds=bnds, constraints=cons, method='SLSQP', options=options)
        solution = pd.Series(index=self.x_data.columns.tolist(), data=res.x)
        solution_with_alpha = solution

        y_estimate = self.x_data.dot(solution)

        solution_without_alpha = solution.drop('alpha')
        solution_without_alpha.sort_values(inplace=True)

        sse = self.y_data.subtract(y_estimate).pow(2.0).sum()
        sst = self.y_data.subtract(self.y_data.mean()).pow(2.0).sum()
        r_square = 1 - sse / sst
        p_value = pd.Series()

        return {
            'solution': solution_without_alpha,
            'solution_with_alpha': solution_with_alpha,
            'sse': sse,
            'alpha': res.x[0],
            'y_estimate_wo_alpha': y_estimate.subtract(res.x[0]),
            'r_square': r_square,
            'p_value': p_value
        }



"""
取数function模块
"""
import hbshare as hbs
import pandas as pd
import datetime
from sqlalchemy import create_engine

sql_params = {
    "ip": "192.168.223.152",
    "user": "readonly",
    "pass": "c24mg2e6",
    "port": "3306",
    "database": "work"
}
engine_params = "mysql+pymysql://{}:{}@{}:{}/{}".format(sql_params['user'], sql_params['pass'],
                                                        sql_params['ip'],
                                                        sql_params['port'], sql_params['database'])


def get_trading_day_list(start_date, end_date, frequency='day'):
    """
    获取日期序列
    """
    sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
        start_date, end_date)
    res = hbs.db_data_query('readonly', sql_script, page_size=5000)
    df = pd.DataFrame(res['data']).rename(
        columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                 "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
    df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
    df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
    df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

    calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

    if frequency == "day":
        trading_day_list = calendar_df[calendar_df['isOpen'] == 1]['calendarDate'].tolist()
    elif frequency == "week":
        trading_day_list = calendar_df[calendar_df['isWeekEnd'] == 1]['calendarDate'].tolist()
    elif frequency == 'month':
        trading_day_list = calendar_df[calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()
    else:
        trading_day_list = calendar_df['calendarDate'].tolist()

    return trading_day_list


def get_fund_nav_from_sql(start_date, end_date, fund_dict):
    """
    获取db的私募基金净值数据
    """
    nav_series_list = []
    for name, fund_id in fund_dict.items():
        sql_script = "SELECT a.jjdm fund_id, b.jzrq TRADEDATE, b.fqdwjz as ADJ_NAV from " \
                     "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                     "and a.jjzt not in ('3') " \
                     "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                     "order by b.jzrq".format(fund_id, start_date, end_date)
        res = hbs.db_data_query("highuser", sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).set_index('TRADEDATE')['ADJ_NAV']
        data.name = name
        nav_series_list.append(data)
    df = pd.concat(nav_series_list, axis=1).sort_index()

    return df


def get_fund_nav_from_work(start_date, end_date, fund_id):
    sql_script = "SELECT name, t_date, nav FROM fund_data where " \
                 "t_date >= {} and t_date <= {} and code = '{}'".format(start_date, end_date, fund_id)
    engine = create_engine(engine_params)
    data = pd.read_sql(sql_script, engine)
    data['t_date'] = data['t_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))

    return data


# -*- coding:utf-8 -*- 
"""
pro init 
Created on 2018/07/01
@author: Jimmy Liu
@group : tushare.pro
@contact: jimmysoa@sina.cn
"""
from hbshare.base import data_api, upass


def hb_api(token='',timeout=300):
    """
    初始化pro API,第一次可以通过ts.set_token('your token')来记录自己的token凭证，临时token可以通过本参数传入
    """
    if token == '' or token is None:
        token = upass.get_token()
    if token is not None and token != '':
        pro = data_api.DataApi(token,timeout)
        return pro
    else:
        raise Exception('api init error.') 




#!/usr/bin/python
#coding:utf-8
from __future__ import division
import time
import pandas as pd
from hbshare.db.simu import cons as ct
import hbshare as hbs

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: corp.py
@time: 2020/9/18 9:59
"""


def get_simu_corp_list_by_keyword(keyword, retry_count=3, pause=0.01, timeout=10):
    """
        根据关键字查询私募机构管理人列表
    :param code:
    :return:
    """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()
        url = ct.HOWBUY_SIMU_CORP_SEARCH % (ct.P_TYPE['https'], ct.DOMAINS['s'], keyword)

        org_js = api.query(url)
        total_count = int(org_js['totalCount'])

        if 'smgsContent' not in org_js:
            status = "未查询到数据"
            raise ValueError(status)

        data = org_js['smgsContent']
        corp_df = pd.DataFrame(data, columns=ct.HOWBUY_SIMU_CORP)
        corp_df['tsname'] = corp_df['tsname'].astype(str)
        corp_df['tsshortName'] = corp_df['tsshortName'].str.replace("#", "").str.replace("$", "").astype(str)
        corp_df['tscode'] = corp_df['tscode'].astype(str)

        return corp_df
    raise IOError(ct.NETWORK_URL_ERROR_MSG)


def get_prod_list_by_corp_code(corp_code, retry_count=3, pause=0.01, timeout=10):
    """
            根据私募机构管理人代码查询机构管理的产品列表
        :param code:
        :return:
        """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()
        url = ct.HOWBUY_SIMU_GET_PROD_LIST_BY_CORP_CODE % (ct.P_TYPE['https'], ct.DOMAINS['hbcgi'], corp_code)

        org_js = api.query(url)
        status_code = str(org_js['code'])
        if status_code != '0000':
            status = str(org_js['desc'])
            raise ValueError(status)

        if 'gljjList' not in org_js['body']:
            status = "未查询到该公司管理产品数据"
            raise ValueError(status)

        data = org_js['body']['gljjList']
        prod_df = pd.DataFrame(data, columns=ct.HOWBUY_SIMU_PROD_LIST)
        # corp_df['tsname'] = corp_df['tsname'].astype(str)
        # corp_df['tsshortName'] = corp_df['tsshortName'].str.replace("#", "").str.replace("$", "").astype(str)
        # corp_df['tscode'] = corp_df['tscode'].astype(str)

        return prod_df
    raise IOError(ct.NETWORK_URL_ERROR_MSG)


# -*- coding: utf-8 -*-

import pandas as pd
import pymysql
import hbshare as hbs
import warnings
warnings.filterwarnings('ignore', category=pymysql.Warning)


class HBDB:
    def __init__(self):
        hbs.set_token("")

    def get_df(self, sql, db, page_size=2000):
        data = hbs.db_data_query(db, sql, page_size=page_size, timeout=120)
        pages = data['pages']
        data = pd.DataFrame(data['data'])
        if pages > 1:
            for page in range(2, pages + 1):
                temp_data = hbs.db_data_query(db, sql, page_size=page_size, page_num=page, timeout=120)
                data = pd.concat([data, pd.DataFrame(temp_data['data'])], axis=0)
        return data

    def read_report_date(self):
        sql = "select distinct(jsrq) as REPORT_DATE FROM st_fund.t_st_gm_gpzh"
        df = self.get_df(sql, db='funduser')
        return df

    def read_cal(self, start, end):
        sql = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ>={0} and JYRQ<={1}".format(start, end)
        df = self.get_df(sql, db='readonly')
        return df

    def read_stock_fund_info(self):
        sql = "select jjdm, jjmc, jjjc, clrq, zzrq, ejfl, kffb from st_fund.t_st_gm_jjxx where jjzt='0' and cpfl='2' and ejfl in ('13', '35', '37')"
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_gptzzjb_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, gptzzjb from st_fund.t_st_gm_zcpz where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_holding_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, ccsz, ccsl, zjbl from st_fund.t_st_gm_gpzh where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_holding_diff_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, zclb, zgblbd, sfsqzcg from st_fund.t_st_gm_jjcgbd where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_holding_semi_diff_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zqdm, zqmc, zzjbl, zjblpm, ggzjzbjbd from st_fund.t_st_gm_gpzhggtj where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_valuation_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zclb, pe, pb, roe, dividend from st_fund.t_st_gm_jjggfg where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=2000)
        return df

    def read_stock_valuation_given_date(self, date):
        sql = "select b.SECUCODE, to_char(a.TRADINGDAY, 'yyyyMMdd') as TRADINGDAY, a.TOTALMV, a.PE, a.PB from hsjy_gg.LC_DIndicesForValuation a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TRADINGDAY, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    def read_star_stock_valuation_given_date(self, date):
        sql = "select b.SECUCODE, to_char(a.TradingDay, 'yyyyMMdd') as TRADINGDAY, a.TotalMV as TOTALMV, a.PETTM as PE, a.PB from hsjy_gg.LC_STIBDIndiForValue a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly')
        return df

    # def read_stock_industry(self):
    #     sql = "select gsdm, xxfbrq, xxly, xyhfbz, ssxy, sfzx, qxrq, yjxydm, yjxymc, ejxydm, ejxymc from st_ashare.t_st_ag_gshyhfb"
    #     df = self.get_df(sql, db='alluser')
    #     return df

    def read_stock_industry(self):
        sql = "select zqdm, flmc, fldm, fljb, hyhfbz, qsrq, jsrq, sfyx from st_fund.t_st_gm_zqhyflb"
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_industry_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zclb, hyhfbz, fldm, flmc, zzjbl, hyzjzbjbd, hyzjzbltlpj, zgpbl, hyzgpbjbd, hyzgpbltlpj, zltgbl, hyzltgbjbd, hyzltgbltlpj from st_fund.t_st_gm_jjhyzhyszb where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_industry_theme(self):
        sql = "select hybh, hyzt, fldm, qsrq, jsrq from st_fund.t_st_gm_hyztpzb"
        df = self.get_df(sql, db='funduser')
        return df

    def read_fund_theme_given_codes(self, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jsrq, zblx, zclb, zgpszb, jsqbd, tlpj from st_fund.t_st_gm_ztcgbd where jjdm in ({0})".format(codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_industry_info(self):
        sql = "select hyhfbz, fldm, flmc, zsdm, qsrq, jsrq, fljb, sfyx from st_market.t_st_zs_hyzsdmdyb"
        df = self.get_df(sql, db='alluser')
        return df

    def read_index_info(self):
        sql = "select a.IndexCode, b.SecuCode, b.ChiName, b.ChiNameAbbr from hsjy_gg.LC_IndexBasicInfo a left join hsjy_gg.SecuMain b on a.IndexCode=b.InnerCode"
        df = self.get_df(sql, db='readonly')
        return df

    def read_index_cons(self, index):
        sql = "select a.IndexCode, b.SecuCode, a.InfoSource, a.EndDate, a.Weight from hsjy_gg.LC_IndexComponentsWeight a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where a.IndexCode='{0}'".format(index)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_fund_style(self):
        sql = "select fund_id, trade_date, type, cap_score, vcg_score, category from st_fund.r_st_gm_equity_score"
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_stock_style(self):
        sql = "select ticker, trade_date, type, cap_score, vcg_score, category from st_fund.r_st_gp_equity_score"
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_barra_style_exposure_given_dates(self, dates):
        dates = '"' + '","'.join(dates) + '"'
        sql = "select ticker, trade_date, size, beta, momentum, resvol, btop, sizenl, liquidity, earnyield, growth, leverage from st_ashare.r_st_barra_style_factor where trade_date in ({0})".format(dates)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_index_daily_k_given_date_and_indexs(self, date, indexs):
        indexs = '"' + '","'.join(indexs) + '"'
        sql = "select scdm, zqdm, zqmc, jyrq, qspj, kpjg, spjg, zgjg, zdjg, zdsl, zdfd, bdfd, cjsl, cjjs, cjbs, ltsz, zsz from st_market.t_st_zs_hqql where jyrq>='{0}' and zqdm in ({1}) ".format(date, indexs)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_fund_nav_given_date_and_codes(self, date, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jzrq, jjjz, ljjz from st_fund.t_st_gm_jjjz where jzrq>='{0}' and jjdm in ({1}) ".format(date, codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_fund_nav_adj_given_date_and_codes(self, date, codes):
        codes = '"' + '","'.join(codes) + '"'
        sql = "select jjdm, jzrq, fqdwjz, hbfh, hbcl, hbdr from st_fund.t_st_gm_rhb where jzrq>='{0}' and jjdm in ({1}) ".format(date, codes)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_finance_data_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.InfoPublDate, 'yyyyMMdd') AS PUBLISH_DATE, to_char(a.EndDate, 'yyyyMMdd') AS END_DATE, ROETTM AS ROE_TTM, ROATTM AS ROA_TTM, ROICTTM AS ROIC_TTM, NetProfitRatioTTM AS NET_PROFIT_RATIO_TTM, GrossIncomeRatioTTM AS GROSS_INCOME_RATIO_TTM, EBITToToAssetsTTM AS EBIT_ASSET_RATIO_TTM from hsjy_gg.LC_MainIndexNew a left join hsjy_gg.SecuMain b on a.CompanyCode=b.CompanyCode where to_char(a.EndDate, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

    def read_stock_market_value_given_date(self, date):
        sql = "select b.SecuCode AS TICKER_SYMBOL, b.ChiNameAbbr AS SEC_SHORT_NAME, to_char(a.TradingDay, 'yyyyMMdd') AS TRADE_DATE, a.TotalMV AS MARKET_VALUE, a.NegotiableMV AS NEG_MARKET_VALUE from hsjy_gg.LC_DIndicesForValuation a left join hsjy_gg.SecuMain b on a.InnerCode=b.InnerCode where to_char(a.TradingDay, 'yyyyMMdd')='{0}'".format(date)
        df = self.get_df(sql, db='readonly', page_size=200000)
        return df

if __name__ == '__main__':
    dates = HBDB().read_report_date()
    barra_style_exposure = HBDB().read_barra_style_exposure_given_dates(fund_zc_wmv['RECENT_TRADE_DATE'].unique().tolist())
    barra_style_exposure.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/barra_style_exposure.hdf', key='table', mode='w')


# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Pro数据接口 
Created on 2020/06/15
@author: meng.lv
@group : hbshare
"""

import simplejson as json
from functools import partial
import requests


class DataApi:

    __token = ''
    __base_url = 'https://data.howbuy.com'

    def __init__(self, token, timeout=300):
        """
        Parameters
        ----------
        token: str
            API接口TOKEN，用于用户认证
        """
        self.__token = token
        if timeout and timeout > 600:
            # 最大600
            timeout = 600
        self.__timeout = timeout

    def query(self, url, method='get', post_data={},params={}):
        headers = {
            "token": self.__token,
            "Referer": 'hbshare'
        }

        if method == 'get':
            res = requests.get(url, headers=headers, timeout=self.__timeout,params=params)
        else:
            res = requests.post(url, json=post_data, headers=headers, timeout=self.__timeout,params=params)
        if res.status_code != 200:
            res.raise_for_status()
        result = json.loads(res.text)
        return result

    def get(self, url):
        headers = {
            "token": self.__token,
            "Referer": 'hbshare'
        }

        res = requests.get(url, headers=headers, timeout=self.__timeout)

        return res.text

    def __getattr__(self, name):
        return partial(self.query, name)

    def __str__(self):
        return u"%s, %s" % (self.__base_url, self.__token)



"""
基差相关数据
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class StockSharePriceIndex:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_share_price_index_futures'

    def get_share_price_index_futures_data(self):
        # ZZ500
        res = w.wsd('000905.SH', 'close', self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch index data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data[0], index=res.Times, columns=res.Codes)
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        index_data = data.copy()
        # index futures
        index_list = ['IC00.CFE', 'IC01.CFE', 'IC02.CFE', 'IC03.CFE']
        res = w.wsd(','.join(index_list), 'settle', self.start_date, self.end_date, '')
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch index_futures data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        future_data = data.copy()

        data = pd.merge(index_data, future_data, on='trade_date')
        data.rename(columns={'000905.SH': 'ZZ500', 'IC00.CFE': 'IC00', 'IC01.CFE': 'IC01',
                             'IC02.CFE': 'IC02', 'IC03.CFE': 'IC03'}, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_share_price_index_futures_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(data, self.table_name)
        else:
            data = self.get_share_price_index_futures_data()
            sql_script = """
                    create table mac_share_price_index_futures(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    ZZ500 decimal(7, 2),
                    IC00 decimal(7, 2),
                    IC01 decimal(7, 2),
                    IC02 decimal(7, 2),
                    IC03 decimal(7, 2)) 
                """
            create_table(self.table_name, sql_script)
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockSharePriceIndex('2021-12-31', '2021-12-31', is_increment=1).get_construct_result()




import numpy as np
import pandas as pd
from hbshare.fe.XZ import db_engine as dbeng
from hbshare.fe.XZ import functionality
from hbshare.fe.mutual_analysis import holdind_based
from hbshare.fe.mutual_analysis import nav_based as nbs
import datetime
from sklearn import preprocessing
from hbshare.fe.factor_analysis import simple_factor_generator as sfg

util=functionality.Untils()
hbdb=dbeng.HBDB()
fre_map={'M':'2101','Q':'2103','HA':'2106'}

def get_jjnav(jjdm_list,date_list,fre):

    fre_con=fre_map[fre]

    jjdm_con=util.list_sql_condition(jjdm_list)
    date_con=util.list_sql_condition(date_list)

    #get the nav ret  for given jjdm and time zone(already times 100)
    sql="select jjdm,jzrq,zbnp from st_fund.t_st_gm_rqjhb where jjdm in ({0}) and jzrq in ({1}) and zbnp!=99999 and zblb='{2}' "\
        .format(jjdm_con,date_con,fre_con)
    navdf=hbdb.db2df(sql, db='funduser')

    return navdf

def get_daily_jjnav(jjdm_list,start_date,end_date):

    jjdm_con=util.list_sql_condition(jjdm_list)

    #get the daily nav ret for given jjdm and time zone
    sql="select jjdm,jzrq,hbdr from st_fund.t_st_gm_rhb where jjdm in ({0}) and jzrq>='{1}' and jzrq<='{2}' and hbdr!=99999  "\
        .format(jjdm_con,start_date,end_date)
    navdf=hbdb.db2df(sql, db='funduser')

    return navdf

def bhar(arr):
    return np.cumprod((arr/100+1)).values[-1]

def pic_grouping(df,pic_title,fre,width=1000,height=1000):

    df['date']=df['date'].astype(str)
    # fre_code=fre_map[fre]
    draw_df=pd.DataFrame()
    df=df.sort_values('date')
    draw_df['date']=df['date'].unique()
    ret_list=[[],[],[],[],[]]
    for i in range(len(draw_df)):
        date=draw_df.iloc[i]['date']
        tempdf=df[df['date']==date].groupby('group').mean()
        for j in range(5):
            ret_list[j].append(tempdf.loc[j]['zbnp'])

    # data_con=until.list_sql_condition(draw_df['date'].tolist())
    # sql = "select zbnp,jyrq from st_market.t_st_zs_rqjhb where zqdm='930950' and zbnp!=99999 and zblb='{1}' and jyrq in ({0}) "\
    #     .format(data_con,fre_code)
    # benchmark_ret=hbdb.db2df(sql,db='alluser')
    # benchmark_ret['jyrq']=benchmark_ret['jyrq'].astype(str)
    # draw_df=pd.merge(draw_df,benchmark_ret,how='left',right_on='jyrq',left_on='date').drop('jyrq',axis=1)
    # draw_df.rename(columns={'zbnp':'benchmark_ret'},inplace=True)
    draw_df = pd.merge(draw_df, df.groupby('date').mean()[['zbnp']], how='left', right_index=True, left_on='date')
    draw_df.rename(columns={'zbnp': 'benchmark_ret'}, inplace=True)
    for i in range(5):
        draw_df['group_'+str(i)]=ret_list[i]
        draw_df['group_'+str(i)]=draw_df['group_'+str(i)].rolling(len(draw_df), 1).apply(bhar)

    draw_df['benchmark_ret']=draw_df['benchmark_ret'].rolling(len(draw_df), 1).apply(bhar)

    draw_df['date']=draw_df['date'].astype(str)
    draw_df.set_index('date',inplace=True)

    plot=functionality.Plot(width,height)
    plot.plotly_line_style(draw_df,pic_title)
    ext_ret = 1+np.max([draw_df.iloc[-1]['group_4'] - draw_df.iloc[-1]['benchmark_ret']
                     , draw_df.iloc[-1]['group_0'] - draw_df.iloc[-1]['benchmark_ret']])
    if(fre=='M'):
        mi=len(draw_df)/12
    elif(fre=='Q'):
        mi = len(draw_df) / 4
    elif(fre=='HA'):
        mi = len(draw_df) / 2
    else:
        print('input frequence is not supported, only M,Q,HY is supported')
        raise Exception

    ext_ret =np.power(ext_ret, 1/mi) - 1
    print('annually ext ret for best group is {}%'.format(ext_ret*100))

def get_stock_ret(ticker_list,date,fre):

    ticker_con = util.list_sql_condition(ticker_list)
    sql = "select jjdm,jzrq,zbnp from st_fund.t_st_gm_rqjhb where zblb='{2}' and jjdm in ({0}) and jzrq ='{1}' and zbnp!=99999 " \
        .format(ticker_con, date, fre)
    retdf = hbdb.db2df(sql, db='funduser')

    return retdf

def grouping(groupdf,retdf,factor_name,col_list,factor_date,ret_date=None):

    if(ret_date is None):
        ret_date=''

    gap = int(np.floor(len(retdf) * 0.2))
    retdf = retdf[col_list+[ factor_name]]
    for group in range(2):
        tempgroup = retdf.sort_values(factor_name)[col_list].iloc[(group) * gap:(1 + group) * gap]
        tempgroup['group'] = group
        tempgroup['date'] = factor_date
        tempgroup['ret_date'] = ret_date
        groupdf = pd.concat([groupdf, tempgroup], axis=0)

        if (group == 0):
            tempgroup = retdf.sort_values(factor_name)[col_list].iloc[-1 * (1 + group) * gap:]
        else:
            tempgroup = retdf.sort_values(factor_name)[col_list].iloc[
                        -1 * (1 + group) * gap:-1 * (group) * gap]
        tempgroup['group'] = 4 - group
        tempgroup['date'] = factor_date
        tempgroup['ret_date'] = ret_date
        groupdf = pd.concat([groupdf, tempgroup], axis=0)

    tempgroup = retdf.sort_values(factor_name)[col_list].iloc[
                2 * gap:-2 * gap]
    tempgroup['group'] = 2
    tempgroup['date'] = factor_date
    tempgroup['ret_date'] = ret_date
    groupdf = pd.concat([groupdf, tempgroup], axis=0)

    return groupdf

def single_factor_ic(factor_df,factor_name,fre,fund_flag=True):

    date_list = factor_df['date'].unique().tolist()
    date_list.sort()
    ic_df=pd.DataFrame()
    # ic_df['date']=date_list[0:-1]
    ic_list=[]
    ic_date=[]
    fre=fre_map[fre]
    groupdf=pd.DataFrame()

    if(fund_flag):
        # ticker_ret=pd.DataFrame()
        for i in range(len(date_list)-1):
            factor_date=date_list[i]
            #ret_date = (datetime.datetime.strptime(factor_date, '%Y%m%d')+ datetime.timedelta(days=91)).strftime('%Y%m%d')

            ret_date=date_list[i+1]

            #get the stock return
            ticker_list = factor_df[factor_df['date']==factor_date]['jjdm'].unique().tolist()
            retdf=get_stock_ret(ticker_list,ret_date,fre)

            #to make sure that the ic calculation is robust, add a requiement of the number of funds in ic calculations
            if(len(retdf)<400):
                continue

            ic_date.append(factor_date)

            retdf['jzrq']=retdf['jzrq'].astype(str)
            retdf=pd.merge(retdf,factor_df[factor_df['date']==factor_date],how='inner',on='jjdm')
            #calculate the spearman correlation between fund return and factors
            ic_list.append(retdf[['zbnp',factor_name]].corr(method='spearman')['zbnp'][1])

            #divide the groups
            groupdf=grouping(groupdf,retdf,factor_name,['jjdm', 'zbnp'],factor_date,ret_date)

        #
        i+=1
        factor_date = date_list[i]
        retdf=factor_df[factor_df['date'] == factor_date]
        groupdf2 = grouping(groupdf[['jjdm','group','date','ret_date']], retdf, factor_name,['jjdm'], factor_date)

    ic_df['ic']=ic_list
    ic_df['date']=ic_date

    print('\n the average ic of {3} is {0}, the pro that ic >0 is {1},the ic std is {4},the IR of the factor is{2} '
          .format(ic_df['ic'].describe()['mean'],
                  sum(ic_df['ic']>0)/len(ic_df),
                  ic_df['ic'].describe()['mean']/ic_df['ic'].describe()['std'],factor_name,
                  ic_df['ic'].describe()['std']))

    return groupdf,groupdf2.drop('ret_date',axis=1)

def factor_narrow_level(groupdf,fre):

    fre_code=fre_map[fre]
    data_con=util.list_sql_condition(groupdf['date'].unique().tolist())
    sql = "select zbnp,jyrq from st_market.t_st_zs_rqjhb where zqdm='930950' and zbnp!=99999 and zblb='{1}' and jyrq in ({0}) " \
        .format(data_con, fre_code)
    benchmark_ret = hbdb.db2df(sql, db='alluser')
    benchmark_ret['jyrq']=benchmark_ret['jyrq'].astype(str)


    ret_df=pd.DataFrame()
    for date in groupdf['date'].unique().tolist():
        tempdf=get_stock_ret(groupdf[groupdf['date']==date]['jjdm'].tolist(),date, fre_map['HA'])
        ret_df=pd.concat([ret_df,tempdf],axis=0)

    ret_df['jzrq']=ret_df['jzrq'].astype(str)
    groupdf=pd.merge(groupdf,ret_df,how='left',left_on=['jjdm','date'],right_on=['jjdm','jzrq']).drop('jzrq',axis=1)
    groupdf=pd.merge(groupdf,benchmark_ret,how='left',left_on='date',right_on='jyrq').drop('jyrq',axis=1)
    groupdf['zbnp']=groupdf['zbnp_x']-groupdf['zbnp_y']
    groupdf.drop(['zbnp_y','zbnp_x'],axis=1,inplace=True)

    factor_narrow_df=pd.DataFrame()
    factor_narrow_df['date']=groupdf['date'].unique()

    for group in [0,4]:
        lsd = []
        std=[]
        corr=[]
        ext_ret=[]
        for date in groupdf['date'].unique():
            tempgroupdf = groupdf[(groupdf['group'] == group)&(groupdf['date'] == date)]
            lsd.append(tempgroupdf['zbnp'].std() / tempgroupdf['zbnp'].mean())

            #get the fund daily returns for the last 60 days
            pre60date=  (datetime.datetime.strptime(date, '%Y%m%d')-datetime.timedelta(days=60)).strftime('%Y%m%d')
            navdf=get_daily_jjnav(tempgroupdf['jjdm'].unique().tolist(),pre60date,date)

            # calculate the factor ret vol for last60 days
            std.append(navdf.groupby('jzrq').mean().std().values[0])

            # calculate the corr insiede the group
            tempdf=navdf.groupby('jzrq').mean().reset_index()
            tempdf['jjdm']='avg'
            navdf=pd.concat([navdf,tempdf],axis=0)
            corr.append(np.mean(navdf.groupby(['jzrq','jjdm']).sum().unstack().corr()[('hbdr',    'avg')].tolist()[0:-1]))

            ext_ret.append(tempgroupdf['zbnp'].mean())

        #z score
        lsd=preprocessing.scale(lsd)
        corr = preprocessing.scale(corr)
        std = preprocessing.scale(std)
        ext_ret=preprocessing.scale(ext_ret)

        factor_narrow_df['lsd_'+str(group)]=lsd
        factor_narrow_df['corr_' + str(group)] = corr
        factor_narrow_df['std_' + str(group)] = std
        factor_narrow_df['ext_ret_' + str(group)] = ext_ret

    factor_narrow_df.set_index('date', inplace=True)
    factor_narrow_df=factor_narrow_df.sum(axis=1).to_frame('factor_narrow_lv')
    return  factor_narrow_df

if __name__ == '__main__':

    localdb=dbeng.PrvFunDB().engine
    bra = holdind_based.Barra_analysis()
    brison=holdind_based.Brinson_ability()


    #factor: hsl
    factor_name='hsl'

    raw_df=sfg.read_factors('factor_hsl')
    groupdf,factrodf=single_factor_ic(raw_df,factor_name=factor_name,fre='HA',fund_flag=True)

    pic_grouping(groupdf, factor_name, 'HA')

    # factor_narrow_lv = factor_narrow_level(factrodf, 'HA')
    # plot = functionality.Plot(2000, 1000)
    # plot.plotly_line_style(factor_narrow_lv, '因子拥挤度')


    # # # new joinner  return factor ic test
    # factor_name='t1_ret'
    #
    # raw_df=bra.factorlize_new_joinner(factor_name)
    # groupdf,factrodf=single_factor_ic(raw_df,factor_name='new_join_'+factor_name,fre='Q',fund_flag=True)
    #
    # pic_grouping(groupdf, factor_name, 'Q')
    #
    # factor_narrow_lv = factor_narrow_level(factrodf, 'Q')
    # plot = functionality.Plot(2000, 1000)
    # plot.plotly_line_style(factor_narrow_lv, '因子拥挤度')

    #
    # # #pess return as factor
    # factor_name='ext_ret'
    # sr=nbs.Scenario_return()
    # raw_df = sr.factorlize_ret(factor_name)
    # groupdf,factrodf = single_factor_ic(raw_df, factor_name=factor_name, fre='M', fund_flag=True)
    # pic_grouping(groupdf, '超额月度',fre='M')
    # factrodf['date']=factrodf['date'].astype(str)
    # factor_narrow_lv = factor_narrow_level(factrodf, 'Q')
    # plot = functionality.Plot(2000, 1000)
    # plot.plotly_line_style(factor_narrow_lv, '因子拥挤度')
    #
    # factor_name='pes_ext_ret'
    #raw_df=nbs.factorlize_ret(factor_name,fre='Q')
    # groupdf=single_factor_ic(raw_df, factor_name=factor_name, fre='Q',fund_flag=True)
    # grouping(groupdf,'逆境季度')

    # for factor_name in ['short_term_equity','long_term_equity','short_term_sector',
    #                     'long_term_sector','short_term_trading','long_term_trading',
    #                     'short_term_asset','long_term_asset']:
    #
    #     raw_df=brison.factorlize_brinson(factor_name)
    #     groupdf,factrodf=single_factor_ic(raw_df, factor_name=factor_name, fre='HA',fund_flag=True)
    #
    #     factor_narrow_lv=factor_narrow_level(factrodf,'HA')
    #
    #
    #     pic_grouping(groupdf, factor_name, 'HA')
    #     plot = functionality.Plot(1000, 1000)
    #     plot.plotly_line_style(factor_narrow_lv, '因子拥挤度')








"""
指数指增主动风格分析
"""
import hbshare as hbs
import pandas as pd
import os
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params
from datetime import datetime
import pyecharts.options as opts
from pyecharts.globals import ThemeType
from pyecharts.charts import Line
# from pyecharts.charts import Grid, Tab
# from hbshare.rm_associated.util.plot_util import draw_active_exposure_series

style_name_list = [
    "size", "beta", "momentum", "earnyield", "resvol", "growth", "btop", "leverage", "liquidity", "sizenl"]
path = r'D:\kevin\risk_model_jy\RiskModel\data'


class IndexActiveStyleCalculator:
    def __init__(self, fund_id, benchmark_id, start_date, end_date):
        self.fund_id = fund_id
        self.benchmark_id = benchmark_id
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]

        df = df[df['isMonthEnd'] == 1].copy()
        trading_day_list = [x for x in sorted(df['calendarDate'].unique()) if x[4:6] in ['06', '12']]

        return trading_day_list

    def _load_portfolio_weight(self):
        sql_script = "SELECT JJDM, JSRQ, ZQDM, ZJBL FROM funddb.GPZH WHERE JJDM = '{}' and GGRQ >= {} " \
                     "and GGRQ <= {}".format(self.fund_id, self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(
            columns={"JSRQ": "endDate", "ZQDM": "ticker", "ZJBL": "weight"})
        date_list = [x for x in sorted(data['endDate'].unique()) if x[4:6] not in ['03', '09']]
        trading_day_list = [
            self.calendar_df[(self.calendar_df['calendarDate'] <= x) & (self.calendar_df['isOpen'] == 1)]
            ['calendarDate'].unique()[-1] for x in date_list]
        map_dict = dict(zip(date_list, trading_day_list))
        data['endDate'] = data['endDate'].replace(map_dict)

        portfolio_weight_series_dict = {}
        for date in trading_day_list:
            portfolio_weight_series_dict[date] = data[data['endDate'] == date].set_index('ticker')['weight'] / 100.

        return portfolio_weight_series_dict

    def _load_benchmark_weight(self, portfolio_weight_series_dict):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
            self.benchmark_id)
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code = index_info.set_index('SECUCODE').loc[self.benchmark_id, 'INNERCODE']
        trading_day_list = sorted(portfolio_weight_series_dict.keys())
        benchmark_weight_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
            benchmark_weight_series_dict[date] = weight_df.set_index(
                'consTickerSymbol')['weight'] / 100.

        return benchmark_weight_series_dict

    @staticmethod
    def _load_style_factor_exposure(portfolio_weight_series_dict):
        style_factor_exposure_dict = dict()
        for date in portfolio_weight_series_dict.keys():
            data_path = os.path.join(path, r'zzqz_sw\style_factor')
            exposure_df = pd.read_csv(
                os.path.join(data_path, '{0}.csv'.format(date)), dtype={"ticker": str}).set_index(
                'ticker')[style_name_list]
            style_factor_exposure_dict[date] = exposure_df

        return style_factor_exposure_dict

    def _load_data(self):
        trading_day_list = self._load_calendar()
        if len(trading_day_list) < 2:
            return
        portfolio_weight_series_dict = self._load_portfolio_weight()
        benchmark_weight_series_dict = self._load_benchmark_weight(portfolio_weight_series_dict)
        style_factor_exposure_series_dict = self._load_style_factor_exposure(portfolio_weight_series_dict)

        data_param = {
            "portfolio_weight_series_dict": portfolio_weight_series_dict,
            "benchmark_weight_series_dict": benchmark_weight_series_dict,
            "style_factor_exposure_series_dict": style_factor_exposure_series_dict
        }

        self.data_param = data_param

    def get_construct_result(self):
        portfolio_weight_series_dict = self.data_param.get('portfolio_weight_series_dict')
        benchmark_weight_series_dict = self.data_param.get('benchmark_weight_series_dict')
        style_factor_exposure_series_dict = self.data_param.get('style_factor_exposure_series_dict')

        exposure_list = []
        for date in portfolio_weight_series_dict.keys():
            portfolio_weight_series = portfolio_weight_series_dict[date]
            benchmark_weight_series = benchmark_weight_series_dict[date]
            style_factor_exposure_series = style_factor_exposure_series_dict[date]

            idx = portfolio_weight_series.index.union(benchmark_weight_series.index).intersection(
                style_factor_exposure_series.index)

            portfolio_weight_series = portfolio_weight_series.reindex(idx).fillna(0.)
            benchmark_weight_series = benchmark_weight_series.reindex(idx).fillna(0.)
            style_factor_exposure_series = style_factor_exposure_series.reindex(idx)

            portfolio_expo = style_factor_exposure_series.T.dot(portfolio_weight_series)
            benchmark_expo = style_factor_exposure_series.T.dot(benchmark_weight_series)
            style_expo = pd.concat([portfolio_expo.to_frame('port'), benchmark_expo.to_frame('bm')], axis=1)
            style_expo['active'] = style_expo['port'] - style_expo['bm']
            style_expo['style'] = style_expo.index
            style_expo['date'] = date
            style_expo.reset_index(drop=True, inplace=True)

            exposure_list.append(style_expo)

        exposure_df = pd.concat(exposure_list, axis=0)

        return exposure_df


class EquityIndexStyleCalculator:
    def __init__(self, start_date, end_date, benchmark_id, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.benchmark_id = benchmark_id
        self.is_increment = is_increment
        self.table_name = 'stock_index_style_exposure'
        self._load_data()

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df[['calendarDate', 'isOpen', 'isWeekEnd', 'isMonthEnd']]
        trading_day_list = self.calendar_df[self.calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list

    def _load_benchmark_weight(self, trading_day_list):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
            self.benchmark_id)
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code = index_info.set_index('SECUCODE').loc[self.benchmark_id, 'INNERCODE']
        benchmark_weight_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
            benchmark_weight_series_dict[date] = weight_df.set_index(
                'consTickerSymbol')['weight'] / 100.

        return benchmark_weight_series_dict

    @staticmethod
    def _load_style_exposure(trading_day_list):
        style_factor_exposure_series_dict = dict()
        for date in trading_day_list:
            data_path = os.path.join(path, r'zzqz_sw\style_factor')
            exposure_df = pd.read_csv(
                os.path.join(data_path, '{0}.csv'.format(date)), dtype={"ticker": str}).set_index(
                'ticker')[style_name_list]
            # sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
            # res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            # exposure_df = pd.DataFrame(res['data']).set_index('ticker')[style_name_list]
            style_factor_exposure_series_dict[date] = exposure_df

        return style_factor_exposure_series_dict

    def _load_data(self):
        trading_day_list = self._load_calendar()
        benchmark_weight_series_dict = self._load_benchmark_weight(trading_day_list)
        style_factor_exposure_series_dict = self._load_style_exposure(trading_day_list)

        self.data_param = {"benchmark_weight_series_dict": benchmark_weight_series_dict,
                           "style_factor_exposure_series_dict": style_factor_exposure_series_dict}

    def get_benchmark_style_exposure(self):
        benchmark_weight_series_dict = self.data_param.get('benchmark_weight_series_dict')
        style_factor_exposure_series_dict = self.data_param.get('style_factor_exposure_series_dict')

        exposure_list = []
        for date in benchmark_weight_series_dict.keys():
            benchmark_weight_series = benchmark_weight_series_dict[date]
            style_factor_exposure_series = style_factor_exposure_series_dict[date]

            idx = benchmark_weight_series.index.intersection(style_factor_exposure_series.index)

            benchmark_weight_series = benchmark_weight_series.reindex(idx)
            style_factor_exposure_series = style_factor_exposure_series.reindex(idx)
            exposure_series = style_factor_exposure_series.T.dot(benchmark_weight_series)
            exposure_list.append(exposure_series.to_frame(date))

        exposure_df = pd.concat(exposure_list, axis=1).T
        exposure_df['trade_date'] = exposure_df.index
        exposure_df = pd.melt(exposure_df, id_vars=['trade_date'], var_name='factor_name', value_name='exposure')
        exposure_df['benchmark_id'] = self.benchmark_id

        return exposure_df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_benchmark_style_exposure()
            sql_script = "delete from {} where trade_date in ({}) and benchmark_id = {}".format(
                self.table_name, ','.join(data['trade_date'].tolist()), self.benchmark_id)
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table stock_index_style_exposure(
                id int auto_increment primary key,
                trade_date date not null,
                factor_name varchar(20),
                exposure decimal(4, 3),
                benchmark_id varchar(10))
            """
            create_table(self.table_name, sql_script)
            data = self.get_benchmark_style_exposure()
            WriteToDB().write_to_db(data, self.table_name)


class EquityIndexStyle:
    def __init__(self, start_date, end_date, benchmark_id):
        self.start_date = start_date
        self.end_date = end_date
        self.benchmark_id = benchmark_id
        self._load_data()

    def _load_data(self):
        sql_script = "SELECT * FROM stock_index_style_exposure where TRADE_DATE >= {} and TRADE_DATE <= {} " \
                     "and BENCHMARK_ID = '{}'".format(self.start_date, self.end_date, self.benchmark_id)
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        data = pd.pivot_table(
            data, index='trade_date', columns='factor_name', values='exposure').sort_index()[style_name_list]

        self.data = data

    def generate_trendency_line(self, title):
        df = self.data.copy()
        line = Line(
            init_opts=opts.InitOpts(
                page_title=title,
                width='1200px',
                height='600px',
                theme=ThemeType.MACARONS
            )
        ).set_global_opts(
            title_opts=opts.TitleOpts(title=title),
            legend_opts=opts.LegendOpts(legend_icon='pin', pos_top='5%'),
            xaxis_opts=opts.AxisOpts(
                type_="category",
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
                axisline_opts=opts.AxisLineOpts(on_zero_axis_index=False),
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                min_interval=0.1,
                name='Index Exposure Series(%)',
                name_location='middle',
                name_gap=45,
                name_textstyle_opts=opts.TextStyleOpts(font_size=16),
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
            datazoom_opts=[
                opts.DataZoomOpts(range_start=0, range_end=100),
                opts.DataZoomOpts(type_="inside")],
        ).add_xaxis(
            xaxis_data=df.index.tolist()
        )

        for style_factor in style_name_list:
            line.add_yaxis(
                series_name=style_factor,
                y_axis=df[style_factor].round(3).tolist(),
                label_opts=opts.LabelOpts(is_show=False),
                is_smooth=True
            )

        return line


if __name__ == '__main__':
    # # 300指增：富国(100038)
    # exposure = IndexActiveStyleCalculator('100038', '000300', '20181220', '20210420').get_construct_result()
    # exposure_300 = exposure.groupby('style').mean().reindex(style_name_list)
    # exposure_300.columns = ['富国300指增风格暴露', '沪深300风格暴露', '主动风格暴露']
    # # 500指增：富国(161017)
    # exposure = IndexActiveStyleCalculator('161017', '000905', '20181220', '20210420').get_construct_result()
    # exposure_500 = exposure.groupby('style').mean().reindex(style_name_list)
    # exposure_500.columns = ['富国500指增风格暴露', '中证500风格暴露', '主动风格暴露']
    # # 300 VS 500
    # index_compare = IndexActiveStyleCalculator('510300', '000905', '20181220', '20210420').get_construct_result()
    # exposure_compare = index_compare.groupby('style').mean().reindex(style_name_list)
    # exposure_compare.columns = ['沪深300风格暴露', '中证500风格暴露', '300相对500主动风格暴露']
    #
    # tab = Tab()
    # grid1 = Grid(init_opts=opts.InitOpts(width="1200px", height="800px", theme=ThemeType.WESTEROS))
    # bar1 = draw_active_exposure_series(exposure_300, '300指增主动风格暴露')
    # grid1.add(bar1, grid_opts=opts.GridOpts(pos_bottom='30%'), is_control_axis_index=True)
    # tab.add(grid1, '300指增主动风格暴露')
    # grid2 = Grid(init_opts=opts.InitOpts(width="1200px", height="800px", theme=ThemeType.WESTEROS))
    # bar2 = draw_active_exposure_series(exposure_500, '500指增主动风格暴露')
    # grid2.add(bar2, grid_opts=opts.GridOpts(pos_bottom='30%'), is_control_axis_index=True)
    # tab.add(grid2, '500指增主动风格暴露')
    # grid3 = Grid(init_opts=opts.InitOpts(width="1200px", height="800px", theme=ThemeType.WESTEROS))
    # bar3 = draw_active_exposure_series(exposure_compare, '沪深300相对中证500主动风格暴露')
    # grid3.add(bar3, grid_opts=opts.GridOpts(pos_bottom='30%'), is_control_axis_index=True)
    # tab.add(grid3, '沪深300相对中证500主动风格暴露')
    #
    # tab.render('D:\\kevin\\宽基指数风格暴露.html')

    # EquityIndexStyleCalculator('20201220', '20210804', '000300', is_increment=1).get_construct_result()
    exposure = IndexActiveStyleCalculator('100038', '000300', '20191220', '20210420').get_construct_result()
    print(123)
    # pic = EquityIndexStyle('20111220', '20210804', '000852').generate_trendency_line(title='宽基指数风格暴露走势图')
    # pic.render('D:\\kevin\\123.html')


"""
2021下半年FOF和代销产品净值风格归因
"""
import os
import pandas as pd
from datetime import datetime
import hbshare as hbs
from MyUtil.data_loader import get_fund_nav_from_work
from hbshare.fe.common.util.regressions import Regression


fund_map_dict = {
    "启林中证500指数增强1号": "SGY379",
    "量锐62号": "SGR954",
    "明汯量化中小盘增强1号": "SGG585",
    "明汯价值成长1期3号": "SEE194",
    "天演赛能": "P22984",
    "衍复指增三号": "SJH866",
    "铭量中证500增强1号": "SLY644",
    "因诺聚配中证500指数增强": "SGX346"
}


if __name__ == '__main__':
    start_date = '20210101'
    end_date = '20211231'
    data = get_fund_nav_from_work(start_date, end_date, 'SGG585')
    print(data)